# Generated by roxygen2: do not edit by hand

S3method("[<-","mixtime::mt_time")
S3method("[[<-","mixtime::mt_time")
S3method(Summary,"mixtime::mixtime")
S3method(all.equal,"mixtime::mt_time")
S3method(base::format,mixtime_interval)
S3method(base::format,mt_naive)
S3method(base::print,mixtime_interval)
S3method(base::print,mt_calendar)
S3method(seq,"mixtime::mixtime")
S3method(seq,"mixtime::mt_time")
S3method(vec_cast,mixtime.yearmonth)
S3method(vec_cast,mixtime.yearquarter)
S3method(vec_cast,mixtime.yearweek)
S3method(vec_proxy,mixtime_interval)
export(as_mixtime)
export(cal_gregorian)
export(cal_isoweek)
export(cal_sym454)
export(cal_time_civil)
export(cal_time_lunar)
export(cal_time_lunar_synodic)
export(cal_time_solar)
export(cal_time_solar_transit)
export(chronon_cardinality)
export(chronon_cardinality_fixed)
export(chronon_common)
export(chronon_divmod)
export(chronon_epoch)
export(chronon_format_attr)
export(chronon_format_cyclical)
export(chronon_format_duration)
export(chronon_format_linear)
export(chronon_parse_cyclical)
export(chronon_parse_linear)
export(circsum)
export(class_mixtime)
export(cyc)
export(cyclical_labels)
export(cyclical_labels_format)
export(cyclical_labels_parse)
export(cyclical_time)
export(date)
export(datetime)
export(day_of_month)
export(day_of_week)
export(day_of_year)
export(days)
export(duration)
export(hours)
export(is_mixtime)
export(is_time_cyclical)
export(is_time_duration)
export(is_time_linear)
export(label_scheme)
export(lin)
export(linear_labels)
export(linear_labels_format)
export(linear_labels_parse)
export(linear_time)
export(loc_altitude)
export(loc_latitude)
export(loc_longitude)
export(milliseconds)
export(minutes)
export(mixtime)
export(month_of_year)
export(months)
export(mt_cyclical)
export(mt_duration)
export(mt_linear)
export(mt_loc_unit)
export(mt_time)
export(mt_tz_unit)
export(mt_unit)
export(new_calendar)
export(new_cyclical_time_fn)
export(new_duration_fn)
export(new_linear_time_fn)
export(new_mixtime)
export(new_time)
export(new_time_unit)
export(parse_format)
export(quarters)
export(seconds)
export(time_calendar)
export(time_ceiling)
export(time_chronon)
export(time_components)
export(time_compose)
export(time_cycle)
export(time_floor)
export(time_is_complete_at)
export(time_is_cyclical)
export(time_is_determinate_at)
export(time_is_duration)
export(time_is_linear)
export(time_of_day)
export(time_parse)
export(time_round)
export(time_unit_abbr)
export(time_unit_full)
export(time_unit_plural)
export(tz_abbreviation)
export(tz_name)
export(tz_offset)
export(tz_transitions)
export(vocab_table)
export(week_of_year)
export(weeks)
export(year)
export(yearmonth)
export(yearquarter)
export(years)
export(yearweek)
import(lifecycle)
import(vctrs)
importFrom(S7,"S7_data<-")
importFrom(S7,"method<-")
importFrom(S7,"props<-")
importFrom(S7,S7_data)
importFrom(S7,S7_inherits)
importFrom(S7,class_any)
importFrom(S7,method)
importFrom(S7,new_class)
importFrom(S7,new_generic)
importFrom(S7,new_property)
importFrom(S7,props)
importFrom(S7,super)
importFrom(rlang,enexpr)
importFrom(rlang,enexprs)
importFrom(rlang,enquo)
importFrom(rlang,enquos)
importFrom(rlang,ensym)
importFrom(rlang,ensyms)
importFrom(rlang,eval_tidy)
importFrom(rlang,expr)
importFrom(rlang,exprs)
importFrom(rlang,list2)
importFrom(rlang,quo)
importFrom(rlang,quos)
importFrom(rlang,sym)
importFrom(rlang,syms)
importFrom(vctrs,vec_math)
importFrom(vctrs,vec_proxy_equal)
importFrom(vctrs,vec_proxy_order)
importFrom(vctrs,vec_restore)
importFrom(vecvec,class_vecvec)
importFrom(vecvec,unvecvec)
importFrom(vecvec,vecvec_apply)
useDynLib(mixtime, .registration = TRUE)
