# Generated by cpp11: do not edit by hand

mt_glue_fmt <- function(input, env) {
  .Call(`_mixtime_mt_glue_fmt`, input, env)
}

approx_lunations_from_utc <- function(unix_times) {
  .Call(`_mixtime_approx_lunations_from_utc`, unix_times)
}

approx_utc_from_lunations <- function(lunation_counts) {
  .Call(`_mixtime_approx_utc_from_lunations`, lunation_counts)
}

approx_lunar_phase_from_utc <- function(unix_times) {
  .Call(`_mixtime_approx_lunar_phase_from_utc`, unix_times)
}

approx_utc_from_lunar_phase <- function(lunation_counts) {
  .Call(`_mixtime_approx_utc_from_lunar_phase`, lunation_counts)
}

approx_solar_seconds_from_utc <- function(unix_times, lat_deg, lon_deg, alt_deg) {
  .Call(`_mixtime_approx_solar_seconds_from_utc`, unix_times, lat_deg, lon_deg, alt_deg)
}

approx_utc_from_solar_seconds <- function(solar_second_counts, lat_deg, lon_deg, alt_deg) {
  .Call(`_mixtime_approx_utc_from_solar_seconds`, solar_second_counts, lat_deg, lon_deg, alt_deg)
}

approx_solar_phase_utc <- function(phase_counts, lat_deg, lon_deg, alt_deg) {
  .Call(`_mixtime_approx_solar_phase_utc`, phase_counts, lat_deg, lon_deg, alt_deg)
}

approx_solar_phase_from_utc <- function(unix_times, lat_deg, lon_deg, alt_deg) {
  .Call(`_mixtime_approx_solar_phase_from_utc`, unix_times, lat_deg, lon_deg, alt_deg)
}

approx_solar_phase_from_solar_seconds <- function(solar_second_counts, lat_deg, lon_deg, alt_deg) {
  .Call(`_mixtime_approx_solar_phase_from_solar_seconds`, solar_second_counts, lat_deg, lon_deg, alt_deg)
}

approx_solar_seconds_from_solar_phase <- function(phase_counts, lat_deg, lon_deg, alt_deg) {
  .Call(`_mixtime_approx_solar_seconds_from_solar_phase`, phase_counts, lat_deg, lon_deg, alt_deg)
}

get_tz_offset <- function(posixct, zone) {
  .Call(`_mixtime_get_tz_offset`, posixct, zone)
}

get_tz_abbreviation <- function(posixct, zone) {
  .Call(`_mixtime_get_tz_abbreviation`, posixct, zone)
}

get_tz_transitions <- function(start_posixct, end_posixct, zone_name) {
  .Call(`_mixtime_get_tz_transitions`, start_posixct, end_posixct, zone_name)
}
