% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/time_chronon.R
\name{time_chronon}
\alias{time_chronon}
\title{Obtain the chronon of a time object}
\usage{
time_chronon(x, ...)
}
\arguments{
\item{x}{A time object (e.g., \link[base:Date]{base::Date}, \link[base:POSIXct]{base::POSIXct}, \code{\link[=linear_time]{linear_time()}}, etc.)}

\item{...}{Additional arguments for methods.}
}
\value{
A time \code{\link[=duration]{duration()}} vector representing the chronon of each
value (e.g., \code{days(1L)}).
}
\description{
This S7 generic function extracts the chronon (the smallest time granule) from a
time object, such as continuous time or cyclical time representations.
}
\examples{

# The chronon of a Date object is 1 day
time_chronon(Sys.Date())

# The chronon of a POSIXct object is 1 second
time_chronon(Sys.time())

# The chronon of a continuous time year and month is 1 month
time_chronon(yearmonth(Sys.Date()))

# The common chronon of a mixed time object is the finest chronon
time_chronon(c(yearmonth(Sys.Date()), Sys.Date()))

}
