% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format.R
\name{format.mt_time}
\alias{format.mt_time}
\alias{format,mt_time-method}
\title{Format mixtime vectors}
\usage{
\S4method{format}{mt_time}(x, ..., attr = TRUE)
}
\arguments{
\item{x}{A mixtime vector.}

\item{...}{Additional arguments for methods, including \code{format}: a
glue-style format string, defaulting to one derived automatically for \code{x}.}

\item{attr}{If \code{TRUE} (default), append attribute information (e.g.
timezone) to the default format.}
}
\value{
A character vector the same length as \code{x}.
}
\description{
Formats a mixtime vector as a character vector, using a glue-style format
string of \code{{lin(...)}}/\code{{cyc(...)}} tokens tailored to the vector's chronon
and cycle. If \code{format} is omitted, a sensible default is derived
automatically. See \code{vignette("time-format-strings")} for the format string
syntax.
}
